STMT = 'STMT'
ID = 'ID'
STRING = 'STRING'
INT = 'INT'
NUMERIC = 'NUMERIC'
NEW_LINE = "NEW_LINE"
OPERATOR = "OPERATOR"
SPECIAL_OPERATOR = "SPECIAL_OPERATOR"
BRACKET = "BRACKET"
SQUARE_BRACKET = "SQUARE_BRACKET"
KEYWORD = "KEYWORD"
PRINT_KEYWORD = "PRINT_KEYWORD"
BSLINT_COMMAND = 'BSLINT_COMMAND'
COMMENT = "COMMENT"
ERROR = "ERROR"
WARNING = "WARNING"
INCREMENT_INDENTATION = 1
DECREMENT_INDENTATION = -1
NO_INDENTATION = 0
SPECIAL_INDENTATION = 2
WARNING_COLOUR = '\033[93m'
PASS_COLOUR = '\033[92m'
END_COLOUR = '\033[0m'
ERROR_COLOUR = '\033[91m'
FILE_COLOUR = '\033[34;4m'
TITLE_COLOUR = '\033[95m'
HEADER_COLOUR = '\033[34;4m'
PENULTIMATE_CHARACTER = -2
CURLY_BRACKET = 'CURLY_BRACKET'
